unit LblEfEds;

{
  This unit implements several property editors for the TLabelEffect
  component, being for the TEffectStyle, TColourScheme, TEffectOption,
  TGraduateOption and TResizeOption types.
  Written by Keith Wood - 17 Feb 1996.

  Plus a component editor for the TLabelEffect component as a whole.
  Allows all of the important properties of this component to be
  manipuated with the effects being immediately visible.
  Written by Keith Wood - 13 Jan 1997.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, LblEffct, StdCtrls, Buttons, DsgnIntf, ExtCtrls,
  TabNotBk, Spin;

type
{******************************************************************************
  TEffectStyle property editor
}

  { The property editor for the TEffectStyle type }
  TEffectStylePropertyEditor = class(TEnumProperty)
  public
    { Public declarations }
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { The form to be displayed when choosing a TEffectStyle value }
  TEffectStylePropEd = class(TForm)
    LabelEffect1: TLabelEffect;    { Each label has a different effect }
    LabelEffect2: TLabelEffect;
    LabelEffect3: TLabelEffect;
    LabelEffect4: TLabelEffect;
    LabelEffect5: TLabelEffect;
    btnOK: TBitBtn;
    BitBtn2: TBitBtn;
    procedure LabelEffectClick(Sender: TObject);
    procedure LabelEffectDblClick(Sender: TObject);
  private
    { Private declarations }
    FEffectStyle: TEffectStyle;
    procedure SetEffectStyle(esStyle: TEffectStyle);
  public
    { Public declarations }
    { Declare a public property to allow communication with the form }
    property EffectStyle: TEffectStyle read FEffectStyle write SetEffectStyle
      default esCustom;
    constructor Create(AOwner:TComponent); override;
  end;

{******************************************************************************
  TColourScheme property editor
}

  { The property editor for the ColourScheme property }
  TColourSchemePropertyEditor = class(TEnumProperty)
  public
    { Public declarations }
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { The form to be displayed when choosing the value of the TColourScheme type }
  TColourSchemePropEd = class(TForm)
    LabelEffect1: TLabelEffect;    { Each label has a different effect }
    LabelEffect2: TLabelEffect;
    LabelEffect3: TLabelEffect;
    LabelEffect4: TLabelEffect;
    LabelEffect5: TLabelEffect;
    btnOK: TBitBtn;
    BitBtn2: TBitBtn;
    procedure LabelEffectClick(Sender: TObject);
    procedure LabelEffectDblClick(Sender: TObject);
  private
    { Private declarations }
    FColourScheme: TColourScheme;
    procedure SetColourScheme(csScheme: TColourScheme);
  public
    { Public declarations }
    { Declare a public property to allow communication with the form }
    property ColourScheme: TColourScheme read FColourScheme write SetColourScheme
      default csCustom;
    constructor Create(AOwner:TComponent); override;
  end;

{******************************************************************************
  TEffectOption property editor
}

  { The property editor for the TEffectOption type }
  TEffectOptionPropertyEditor = class(TEnumProperty)
  public
    { Public declarations }
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { The form to be displayed when choosing a TEffectOption value }
  TEffectOptionPropEd = class(TForm)
    LabelEffect1: TLabelEffect;    { Each label has a different effect }
    LabelEffect2: TLabelEffect;
    LabelEffect3: TLabelEffect;
    LabelEffect4: TLabelEffect;
    btnOK: TBitBtn;
    BitBtn2: TBitBtn;
    Shape1: TShape;
    procedure LabelEffectClick(Sender: TObject);
    procedure LabelEffectDblClick(Sender: TObject);
  private
    { Private declarations }
    FEffectOption: TEffectOption;
    procedure SetEffectOption(eoOption: TEffectOption);
  public
    { Public declarations }
    { Declare a public property to allow communication with the form }
    property EffectOption: TEffectOption read FEffectOption write SetEffectOption
      default eoNormal;
    constructor Create(AOwner:TComponent); override;
  end;

{******************************************************************************
  TGraduateOption property editor
}

  { The property editor for the TGraduateOption type }
  TGraduateOptionPropertyEditor = class(TEnumProperty)
  public
    { Public declarations }
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { The form to be displayed when choosing a TGraduateOption value }
  TGraduateOptionPropEd = class(TForm)
    LabelEffect1: TLabelEffect;    { Each label has a different effect }
    LabelEffect2: TLabelEffect;
    LabelEffect3: TLabelEffect;
    LabelEffect4: TLabelEffect;
    LabelEffect5: TLabelEffect;
    LabelEffect6: TLabelEffect;
    LabelEffect7: TLabelEffect;
    LabelEffect8: TLabelEffect;
    btnOK: TBitBtn;
    BitBtn2: TBitBtn;
    procedure LabelEffectClick(Sender: TObject);
    procedure LabelEffectDblClick(Sender: TObject);
  private
    { Private declarations }
    FGraduateOption: TGraduateOption;
    procedure SetGraduateOption(goOption: TGraduateOption);
  public
    { Public declarations }
    { Declare a public property to allow communication with the form }
    property GraduateOption: TGraduateOption read FGraduateOption
      write SetGraduateOption default goNone;
    constructor Create(AOwner:TComponent); override;
  end;

{******************************************************************************
  TResizeOption property editor
}

  { The property editor for the TResizeOption type }
  TResizeOptionPropertyEditor = class(TEnumProperty)
  public
    { Public declarations }
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  { The form to be displayed when choosing a TResizeOption value }
  TResizeOptionPropEd = class(TForm)
    LabelEffect1: TLabelEffect;    { Each label has a different effect }
    LabelEffect2: TLabelEffect;
    LabelEffect3: TLabelEffect;
    btnOK: TBitBtn;
    BitBtn2: TBitBtn;
    procedure LabelEffectClick(Sender: TObject);
    procedure LabelEffectDblClick(Sender: TObject);
  private
    { Private declarations }
    FResizeOption: TResizeOption;
    procedure SetResizeOption(rsOption: TResizeOption);
  public
    { Public declarations }
    { Declare a public property to allow communication with the form }
    property ResizeOption: TResizeOption read FResizeOption
      write SetResizeOption default rsNone;
    constructor Create(AOwner:TComponent); override;
  end;

{******************************************************************************
  TLabelEffect component editor
}

  { The component editor for the TLabelEffect component }
  TLabelEffectCompEditor = class(TComponentEditor)
  public
    procedure Edit; override;
  end;

{******************************************************************************
  TLabelEffect component editor interface
}

  { The interface form for the TLabelEffect component editor }
  TLabelEffectEditor = class(TForm)
    Label14: TLabel;                        { Overall properties }
    cmbEffectStyle: TComboBox;
    Label15: TLabel;
    cmbColourScheme: TComboBox;
    Label17: TLabel;
    edCaption: TEdit;
    btnFont: TButton;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    tnbComponents: TTabbedNotebook;
    Label11: TLabel;                        { Face page }
    pnlFaceColour: TPanel;
    btnFaceColour: TButton;
    Label16: TLabel;
    cmbGraduateOption: TComboBox;
    Label12: TLabel;
    pnlFaceGraduate: TPanel;
    btnFaceGraduate: TButton;
    Label13: TLabel;
    imgBitmap: TImage;
    btnLoad: TButton;
    btnClear: TButton;
    grpAngle: TGroupBox;
    cbxAngle: TCheckBox;
    imgAngle: TImage;
    spnAngle: TSpinEdit;
    cbxKeepVertical: TCheckBox;
    ragHighlightDirection: TRadioGroup;     { Highlight page }
    Label6: TLabel;
    spnHighlightDepth: TSpinEdit;
    Label8: TLabel;
    pnlHighlightColour: TPanel;
    btnHighlightColour: TButton;
    Label9: TLabel;
    pnlHighlightGraduate: TPanel;
    btnHighlightGraduate: TButton;
    Label7: TLabel;
    cmbHighlightStyle: TComboBox;
    Label10: TLabel;
    cmbHighlightResize: TComboBox;
    ragShadowDirection: TRadioGroup;        { Shadow page }
    Label5: TLabel;
    spnShadowDepth: TSpinEdit;
    Label3: TLabel;
    pnlShadowColour: TPanel;
    btnShadowColour: TButton;
    Label4: TLabel;
    pnlShadowGraduate: TPanel;
    btnShadowGraduate: TButton;
    Label1: TLabel;
    cmbShadowStyle: TComboBox;
    Label2: TLabel;
    cmbShadowResize: TComboBox;
    GroupBox2: TGroupBox;                   { Sample }
    leSample: TLabelEffect;
    dlgFont: TFontDialog;                   { Dialogs }
    dlgOpen: TOpenDialog;
    dlgColour: TColorDialog;
    Label18: TLabel;
    procedure cmbEffectStyleChange(Sender: TObject);        { Overall properties }
    procedure cmbColourSchemeChange(Sender: TObject);
    procedure edCaptionChange(Sender: TObject);
    procedure btnFontClick(Sender: TObject);
    procedure btnFaceColourClick(Sender: TObject);          { Face page }
    procedure cmbGraduateOptionChange(Sender: TObject);
    procedure btnFaceGraduateClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure cbxAngleClick(Sender: TObject);
    procedure imgAngleMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure spnAngleChange(Sender: TObject);
    procedure cbxKeepVerticalClick(Sender: TObject);
    procedure ragHighlightDirectionClick(Sender: TObject);  { Highlight page }
    procedure spnHighlightDepthChange(Sender: TObject);
    procedure btnHighlightColourClick(Sender: TObject);
    procedure btnHighlightGraduateClick(Sender: TObject);
    procedure cmbHighlightStyleChange(Sender: TObject);
    procedure cmbHighlightResizeChange(Sender: TObject);
    procedure ragShadowDirectionClick(Sender: TObject);     { Shadow page }
    procedure spnShadowDepthChange(Sender: TObject);
    procedure btnShadowColourClick(Sender: TObject);
    procedure btnShadowGraduateClick(Sender: TObject);
    procedure cmbShadowStyleChange(Sender: TObject);
    procedure cmbShadowResizeChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);                  { OK }
  private
    { Private declarations }
    leOriginal: TLabelEffect;
    procedure InitControls;
    function ChangeColour(clrOrig: TColor; pnlShow: TPanel): TColor;
  public
    { Public declarations }
    constructor CreateFor(AOwner: TComponent; leLabel: TLabelEffect);
  end;

procedure Register;

implementation

{$R LeEfctEd.DFM}
{$R LeColrEd.DFM}
{$R LeStylEd.DFM}
{$R LeGradEd.DFM}
{$R LeReszEd.DFM}
{$R LeCompEd.DFM}

{ Define the property editors and component editor to Delphi }
procedure Register;
begin
  { Only applies to properties of type TEffectStyle in TLabelEffect component }
  RegisterPropertyEditor(TypeInfo(TEffectStyle), TLabelEffect, '',
    TEffectStylePropertyEditor);
  { Only applies to properties of type TColourScheme in TLabelEffect component }
  RegisterPropertyEditor(TypeInfo(TColourScheme), TLabelEffect, '',
    TColourSchemePropertyEditor);
  { Only applies to properties of type TEffectOption in TLabelEffect component }
  RegisterPropertyEditor(TypeInfo(TEffectOption), TLabelEffect, '',
    TEffectOptionPropertyEditor);
  { Only applies to properties of type TGraduateOption in TLabelEffect component }
  RegisterPropertyEditor(TypeInfo(TGraduateOption), TLabelEffect, '',
    TGraduateOptionPropertyEditor);
  { Only applies to properties of type TResizeOption in TLabelEffect component }
  RegisterPropertyEditor(TypeInfo(TResizeOption), TLabelEffect, '',
    TResizeOptionPropertyEditor);
  { Whole of component editor }
  RegisterComponentEditor(TLabelEffect, TLabelEffectCompEditor);
end;

{******************************************************************************
  TEffectStyle property editor
}

{ Tell Delphi that we now have a dialog box for editing as well }
function TEffectStylePropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList];
end;

{ The procedure invoked when the property is double-clicked }
procedure TEffectStylePropertyEditor.Edit;
var
  EffectStylePropEd: TEffectStylePropEd;
begin
  { Create a dialog box as defined above }
  EffectStylePropEd := TEffectStylePropEd.Create(Application);
  try
    { Initialise with the current style }
    EffectStylePropEd.EffectStyle := TEffectStyle(GetOrdValue);
    if EffectStylePropEd.ShowModal = mrOK then
      { If OK then update with the new value }
      SetOrdValue(Ord(EffectStylePropEd.EffectStyle));
  finally
    EffectStylePropEd.Free;    { Tidy up }
  end;
end;

{ Initialise default values for internal fields }
constructor TEffectStylePropEd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEffectStyle := esCustom;
end;

{ Set the EffectStyle property of the form and highlight appropriate label }
procedure TEffectStylePropEd.SetEffectStyle(esStyle: TEffectStyle);
var
  i: Integer;
begin
  if FEffectStyle <> esStyle then
  begin
    FEffectStyle := esStyle;
    for i := 0 to ComponentCount -1 do
      if Components[i] is TLabelEffect then
        if TLabelEffect(Components[i]).EffectStyle = esStyle then
          TLabelEffect(Components[i]).Transparent := False
        else
          TLabelEffect(Components[i]).Transparent := True;
  end;
end;

{ Set the form's EffectStyle property based on the value of the label selected }
procedure TEffectStylePropEd.LabelEffectClick(Sender: TObject);
begin
  EffectStyle := (Sender as TLabelEffect).EffectStyle;
end;

{ As above and exit }
procedure TEffectStylePropEd.LabelEffectDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

{******************************************************************************
  TColourScheme property editor
}

{ Tell Delphi that we now have a dialog box for editing as well }
function TColourSchemePropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList];
end;

{ The procedure invoked when the property is double-clicked }
procedure TColourSchemePropertyEditor.Edit;
var
  ColourSchemePropEd: TColourSchemePropEd;
begin
  { Create a dialog box as defined above }
  ColourSchemePropEd := TColourSchemePropEd.Create(Application);
  try
    { Initialise with the current style }
    ColourSchemePropEd.ColourScheme := TColourScheme(GetOrdValue);
    if ColourSchemePropEd.ShowModal = mrOK then
      { If OK then update with the new value }
      SetOrdValue(Ord(ColourSchemePropEd.ColourScheme));
  finally
    ColourSchemePropEd.Free;    { Tidy up }
  end;
end;

{ Initialise default values for internal fields }
constructor TColourSchemePropEd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColourScheme := csCustom;
end;

{ Set the ColourScheme property of the form and highlight appropriate label }
procedure TColourSchemePropEd.SetColourScheme(csScheme: TColourScheme);
var
  i: Integer;
begin
  if FColourScheme <> csScheme then
  begin
    FColourScheme := csScheme;
    for i := 0 to ComponentCount -1 do
      if Components[i] is TLabelEffect then
        if TLabelEffect(Components[i]).ColourScheme = csScheme then
          TLabelEffect(Components[i]).Transparent := False
        else
          TLabelEffect(Components[i]).Transparent := True;
  end;
end;

{ Set the form's ColourScheme property based on the value of the label selected }
procedure TColourSchemePropEd.LabelEffectClick(Sender: TObject);
begin
  ColourScheme := (Sender as TLabelEffect).ColourScheme;
end;

{ As above and exit }
procedure TColourSchemePropEd.LabelEffectDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

{******************************************************************************
  TEffectOption property editor
}

{ Tell Delphi that we now have a dialog box for editing as well }
function TEffectOptionPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList];
end;

{ The procedure invoked when the property is double-clicked }
procedure TEffectOptionPropertyEditor.Edit;
var
  EffectOptionPropEd: TEffectOptionPropEd;
begin
  { Create a dialog box as defined above }
  EffectOptionPropEd := TEffectOptionPropEd.Create(Application);
  try
    { Initialise with the current style }
    EffectOptionPropEd.EffectOption := TEffectOption(GetOrdValue);
    if EffectOptionPropEd.ShowModal = mrOK then
      { If OK then update with the new value }
      SetOrdValue(Ord(EffectOptionPropEd.EffectOption));
  finally
    EffectOptionPropEd.Free;    { Tidy up }
  end;
end;

{ Initialise default values for internal fields }
constructor TEffectOptionPropEd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEffectOption := eoNormal;
end;

{ Set the EffectOption property of the form and highlight appropriate label }
procedure TEffectOptionPropEd.SetEffectOption(eoOption: TEffectOption);
var
  i: Integer;
begin
  FEffectOption := eoOption;
  for i := 0 to ComponentCount -1 do
    if Components[i] is TLabelEffect then
      if TLabelEffect(Components[i]).StyleShadow = eoOption then
        TLabelEffect(Components[i]).Transparent := False
      else
        TLabelEffect(Components[i]).Transparent := True;
end;

{ Set the form's EffectOption property based on the value of the label selected }
procedure TEffectOptionPropEd.LabelEffectClick(Sender: TObject);
begin
  EffectOption := (Sender as TLabelEffect).StyleShadow;
end;

{ As above and exit }
procedure TEffectOptionPropEd.LabelEffectDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

{******************************************************************************
  TGraduateOption property editor
}

{ Tell Delphi that we now have a dialog box for editing as well }
function TGraduateOptionPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList];
end;

{ The procedure invoked when the property is double-clicked }
procedure TGraduateOptionPropertyEditor.Edit;
var
  GraduateOptionPropEd: TGraduateOptionPropEd;
begin
  { Create a dialog box as defined above }
  GraduateOptionPropEd := TGraduateOptionPropEd.Create(Application);
  try
    { Initialise with the current style }
    GraduateOptionPropEd.GraduateOption := TGraduateOption(GetOrdValue);
    if GraduateOptionPropEd.ShowModal = mrOK then
      { If OK then update with the new value }
      SetOrdValue(Ord(GraduateOptionPropEd.GraduateOption));
  finally
    GraduateOptionPropEd.Free;    { Tidy up }
  end;
end;

{ Initialise default values for internal fields }
constructor TGraduateOptionPropEd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGraduateOption := goNone;
end;

{ Set the GraduateOption property of the form and highlight appropriate label }
procedure TGraduateOptionPropEd.SetGraduateOption(goOption: TGraduateOption);
var
  i: Integer;
begin
  FGraduateOption := goOption;
  for i := 0 to ComponentCount -1 do
    if Components[i] is TLabelEffect then
      if TLabelEffect(Components[i]).GraduateFace = goOption then
        TLabelEffect(Components[i]).Transparent := False
      else
        TLabelEffect(Components[i]).Transparent := True;
end;

{ Set the form's GraduateOption property based on the value of the label selected }
procedure TGraduateOptionPropEd.LabelEffectClick(Sender: TObject);
begin
  GraduateOption := (Sender as TLabelEffect).GraduateFace;
end;

{ As above and exit }
procedure TGraduateOptionPropEd.LabelEffectDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

{******************************************************************************
  TResizeOption property editor
}

{ Tell Delphi that we now have a dialog box for editing as well }
function TResizeOptionPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList];
end;

{ The procedure invoked when the property is double-clicked }
procedure TResizeOptionPropertyEditor.Edit;
var
  ResizeOptionPropEd: TResizeOptionPropEd;
begin
  { Create a dialog box as defined above }
  ResizeOptionPropEd := TResizeOptionPropEd.Create(Application);
  try
    { Initialise with the current style }
    ResizeOptionPropEd.ResizeOption := TResizeOption(GetOrdValue);
    if ResizeOptionPropEd.ShowModal = mrOK then
      { If OK then update with the new value }
      SetOrdValue(Ord(ResizeOptionPropEd.ResizeOption));
  finally
    ResizeOptionPropEd.Free;    { Tidy up }
  end;
end;

{ Initialise default values for internal fields }
constructor TResizeOptionPropEd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FResizeOption := rsNone;
end;

{ Set the ResizeOption property of the form and highlight appropriate label }
procedure TResizeOptionPropEd.SetResizeOption(rsOption: TResizeOption);
var
  i: Integer;
begin
  FResizeOption := rsOption;
  for i := 0 to ComponentCount -1 do
    if Components[i] is TLabelEffect then
      if TLabelEffect(Components[i]).ResizeHighlight = rsOption then
        TLabelEffect(Components[i]).Transparent := False
      else
        TLabelEffect(Components[i]).Transparent := True;
end;

{ Set the form's ResizeOption property based on the value of the label selected }
procedure TResizeOptionPropEd.LabelEffectClick(Sender: TObject);
begin
  ResizeOption := (Sender as TLabelEffect).ResizeHighlight;
end;

{ As above and exit }
procedure TResizeOptionPropEd.LabelEffectDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

{******************************************************************************
  TLabelEffect component editor
}

{ Show the editor for the component }
procedure TLabelEffectCompEditor.Edit;
var
  dlgLabelEffectEditor: TLabelEffectEditor;
begin
  dlgLabelEffectEditor :=
    TLabelEffectEditor.CreateFor(Application, TLabelEffect(Component));
  try
    if dlgLabelEffectEditor.ShowModal = mrOK then
    begin
      TLabelEffect(Component).Invalidate;
      Designer.Modified;
    end;
  finally
    dlgLabelEffectEditor.Free;
  end;
end;

{******************************************************************************
  TLabelEffect component editor interface
}

const
  { Convert between TEffectDirections and integers (not straight Ord() mapping) }
  edDirections: array [0..8] of TEffectDirection =
    (edUpLeft, edLeft, edDownLeft, edUp, edNone, edDown, edUpRight, edRight, edDownRight);
  iDirections: array [TEffectDirection] of 0..8 =
    (4, 3, 6, 7, 8, 5, 2, 1, 0);

{ Create a new editor and initialise it for the specified label }
constructor TLabelEffectEditor.CreateFor(AOwner: TComponent; leLabel: TLabelEffect);
begin
  inherited Create(AOwner);

  { Attach to original label }
  leOriginal := leLabel;
  leSample.Assign(leOriginal);
  with imgAngle.Canvas do
  begin
    Brush.Color := clBtnFace;
    FillRect(ClientRect);
    Brush.Color := clWhite;
  end;
  InitControls;
end;

{ Set controls based on properties of sample }
procedure TLabelEffectEditor.InitControls;
const
  clrFont: array [False..True] of TColor = (clGrayText, clWindowText);
begin
  cmbEffectStyle.ItemIndex := Ord(leSample.EffectStyle);
  cmbColourScheme.ItemIndex := Ord(leSample.ColourScheme);
  edCaption.Text := leSample.Caption;
  pnlFaceColour.Color := leSample.ColourFace;
  cmbGraduateOption.ItemIndex := Ord(leSample.GraduateFace);
  pnlFaceGraduate.Color := leSample.GraduateFrom;
  imgBitmap.Picture.Bitmap.Assign(leSample.Bitmap);
  spnAngle.Value := leSample.Angle;
  if leSample.Angle = 0 then
    spnAngle.Value := 360;
  cbxKeepVertical.Checked := leSample.KeepLettersVertical;
  cbxAngle.Checked := (leSample.Angle > 0);
  spnAngle.Enabled := cbxAngle.Checked;
  spnAngle.Font.Color := clrFont[cbxAngle.Checked];
  cbxKeepVertical.Enabled := cbxAngle.Checked;
  with imgAngle.Canvas do
  begin
    Ellipse(0, 0, 89, 89);
    MoveTo(44, 44);
    LineTo(Round(Sin((spnAngle.Value + 90) * PI / 180) * 44) + 44,
      Round(Cos((spnAngle.Value + 90) * PI / 180) * 44) + 44);
  end;

  ragHighlightDirection.ItemIndex := iDirections[leSample.DirectionHighlight];
  spnHighlightDepth.Value := leSample.DepthHighlight;
  pnlHighlightColour.Color := leSample.ColourHighlight;
  pnlHighlightGraduate.Color := leSample.GraduateHighlight;
  cmbHighlightStyle.ItemIndex := Ord(leSample.StyleHighlight);
  cmbHighlightResize.ItemIndex := Ord(leSample.ResizeHighlight);

  ragShadowDirection.ItemIndex := iDirections[leSample.DirectionShadow];
  spnShadowDepth.Value := leSample.DepthShadow;
  pnlShadowColour.Color := leSample.ColourShadow;
  pnlShadowGraduate.Color := leSample.GraduateShadow;
  cmbShadowStyle.ItemIndex := Ord(leSample.StyleShadow);
  cmbShadowResize.ItemIndex := Ord(leSample.ResizeShadow);
end;

{ Present colour dialog and accept change of colour }
function TLabelEffectEditor.ChangeColour(clrOrig: TColor; pnlShow: TPanel): TColor;
begin
  with dlgColour do
  begin
    Color := clrOrig;
    if Execute then
    begin
      Result := Color;
      pnlShow.Color := Color;
    end
    else
      Result := clrOrig;
  end;
end;

{ Change effect style }
procedure TLabelEffectEditor.cmbEffectStyleChange(Sender: TObject);
begin
  leSample.EffectStyle := TEffectStyle(cmbEffectStyle.ItemIndex);
  InitControls;
end;

{ Change colour scheme }
procedure TLabelEffectEditor.cmbColourSchemeChange(Sender: TObject);
begin
  leSample.ColourScheme := TColourScheme(cmbColourScheme.ItemIndex);
  InitControls;
end;

{ Change label caption }
procedure TLabelEffectEditor.edCaptionChange(Sender: TObject);
begin
  leSample.Caption := edCaption.Text;
  InitControls;
end;

{ Change font }
procedure TLabelEffectEditor.btnFontClick(Sender: TObject);
begin
  with dlgFont do
  begin
    Font.Assign(leSample.Font);
    if Execute then
    begin
      leSample.Font.Assign(Font);
      InitControls;
    end;
  end;
end;

{ Change face colour }
procedure TLabelEffectEditor.btnFaceColourClick(Sender: TObject);
begin
  leSample.ColourFace := ChangeColour(leSample.ColourFace, pnlFaceColour);
  InitControls;
end;

{ Change face graduation option }
procedure TLabelEffectEditor.cmbGraduateOptionChange(Sender: TObject);
begin
  leSample.GraduateFace := TGraduateOption(cmbGraduateOption.ItemIndex);
  InitControls;
end;

{ Change face graduation colour }
procedure TLabelEffectEditor.btnFaceGraduateClick(Sender: TObject);
begin
  leSample.GraduateFrom := ChangeColour(leSample.GraduateFrom, pnlFaceGraduate);
  InitControls;
end;

{ Load a new bitmap }
procedure TLabelEffectEditor.btnLoadClick(Sender: TObject);
begin
  with dlgOpen do
    if Execute then
    begin
      imgBitmap.Picture.Bitmap.LoadFromFile(FileName);
      leSample.Bitmap.LoadFromFile(FileName);
      InitControls;
    end;
end;

{ Clear current bitmap }
procedure TLabelEffectEditor.btnClearClick(Sender: TObject);
begin
  imgBitmap.Picture := nil;
  leSample.Bitmap := nil;
  dlgOpen.FileName := '';
  InitControls;
end;

{ Allow/disallow angled text }
procedure TLabelEffectEditor.cbxAngleClick(Sender: TObject);
begin
  with leSample do
    if cbxAngle.Checked then
    begin
      Angle := spnAngle.Value;
      KeepLettersVertical := cbxKeepVertical.Checked;
    end
    else
      Angle := 0;
  InitControls;
end;

{ Select angle graphically }
procedure TLabelEffectEditor.imgAngleMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  iAngle: Integer;
begin
  if not cbxAngle.Checked then
    Exit;

  if X = 44 then
  begin
    if Y > 44 then
      iAngle := 270
    else
      iAngle := 90;
  end
  else
  begin
    iAngle := Round(ArcTan((Y - 44) / (X - 44)) * 180 / PI);
    if X < 44 then
      iAngle := 180 - iAngle
    else if Y < 44 then
      iAngle := -iAngle
    else
      iAngle := 360 - iAngle;
  end;
  spnAngle.Value := iAngle;
end;

{ Change text angle }
procedure TLabelEffectEditor.spnAngleChange(Sender: TObject);
begin
  if cbxAngle.Checked then
  begin
    leSample.Angle := spnAngle.Value;
    InitControls;
  end;
end;

{ Change vertical text flag }
procedure TLabelEffectEditor.cbxKeepVerticalClick(Sender: TObject);
begin
  leSample.KeepLettersVertical := cbxKeepVertical.Checked;
  InitControls;
end;

{ Change highlight direction }
procedure TLabelEffectEditor.ragHighlightDirectionClick(Sender: TObject);
begin
  leSample.DirectionHighlight := edDirections[ragHighlightDirection.ItemIndex];
  InitControls;
end;

{ Change highlight depth }
procedure TLabelEffectEditor.spnHighlightDepthChange(Sender: TObject);
begin
  leSample.DepthHighlight := spnHighlightDepth.Value;
  InitControls;
end;

{ Change highlight colour }
procedure TLabelEffectEditor.btnHighlightColourClick(Sender: TObject);
begin
  leSample.ColourHighlight := ChangeColour(leSample.ColourHighlight, pnlHighlightColour);
  InitControls;
end;

{ Change highlight graduation colour }
procedure TLabelEffectEditor.btnHighlightGraduateClick(Sender: TObject);
begin
  leSample.GraduateHighlight := ChangeColour(leSample.GraduateHighlight, pnlHighlightGraduate);
  InitControls;
end;

{ Change highlight style }
procedure TLabelEffectEditor.cmbHighlightStyleChange(Sender: TObject);
begin
  leSample.StyleHighlight := TEffectOption(cmbHighlightStyle.ItemIndex);
  InitControls;
end;

{ Change highlight resize option }
procedure TLabelEffectEditor.cmbHighlightResizeChange(Sender: TObject);
begin
  leSample.ResizeHighlight := TResizeOption(cmbHighlightResize.ItemIndex);
  InitControls;
end;

{ Change shadow direction }
procedure TLabelEffectEditor.ragShadowDirectionClick(Sender: TObject);
begin
  leSample.DirectionShadow := edDirections[ragShadowDirection.ItemIndex];
  InitControls;
end;

{ Change shadow depth }
procedure TLabelEffectEditor.spnShadowDepthChange(Sender: TObject);
begin
  leSample.DepthShadow := spnShadowDepth.Value;
  InitControls;
end;

{ Change shadow colour }
procedure TLabelEffectEditor.btnShadowColourClick(Sender: TObject);
begin
  leSample.ColourShadow := ChangeColour(leSample.ColourShadow, pnlShadowColour);
  InitControls;
end;

{ Change shadow graduation colour }
procedure TLabelEffectEditor.btnShadowGraduateClick(Sender: TObject);
begin
  leSample.GraduateShadow := ChangeColour(leSample.GraduateShadow, pnlShadowGraduate);
  InitControls;
end;

{ Change shadow style }
procedure TLabelEffectEditor.cmbShadowStyleChange(Sender: TObject);
begin
  leSample.StyleShadow := TEffectOption(cmbShadowStyle.ItemIndex);
  InitControls;
end;

{ Change shadow resize option }
procedure TLabelEffectEditor.cmbShadowResizeChange(Sender: TObject);
begin
  leSample.ResizeShadow := TResizeOption(cmbShadowResize.ItemIndex);
  InitControls;
end;

{ Copy changes back to original label }
procedure TLabelEffectEditor.btnOKClick(Sender: TObject);
begin
  leOriginal.Assign(leSample);
end;

end.
